#include <windows.h>
#include <windowsx.h>
#include <assert.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "lefttime.h"
#include "digit.h"

#define TID_MAIN	1	///< Timer ID : Main

#define NIID_MAIN	0					///< Notify Icon ID : Main
#define WM_USER_NOTIFYICON	WM_USER+1	///< Notify window message of notify icon
#define STR_NOTIFYICON	"The Left Time"	///< Popup string of the notify icon

#define STR_MENU_CLOSE			"(&C)"			///< Menu command string : Close
#define MID_CLOSE			0							///< Menu command id     : Close
#define STR_MENU_STAYONTOP		"őOʂɕ\\(&S)"		///< Menu command string : Stay on top
#define MID_STAYONTOP		1							///< Menu command id     : Stay on top
#define STR_MENU_TRANSPARENT	"\\(&T)"		///< Menu command string : Transparent
#define MID_TRANSPARENT		2							///< Menu command id     : Transparent
#define STR_MENU_MILLISECONDS	"Z`b\\(&M)"	///< Menu command string : Milliseconds
#define MID_MILLISECONDS	3							///< Menu command id     : Milliseconds
#define STR_MENU_SHOWTOPZEROS	"ŏ0\\(&M)"		///< Menu command string : Show Top Zeros
#define MID_SHOWTOPZEROS	4							///< Menu command id     : Show Top Zeros
#define STR_MENU_VERSION		"o[W(&A)..."	///< Menu command string : Version
#define MID_VERSION			5							///< Menu command id     : Version

#define STR_VERSION_INFO	"The Left Time; tells your left time.\nVersion 1.0.0\n(C)2002 junnno(N.Inoue)\nAdapted by ݂CAT"
#define STR_VERSION_CAPTION	"The Left Time"

#define STR_INI_APPNAME		"TheLeftTime"	///< Section of the initial settings file
#define STR_INI_KEY_LEFT 	"left"
#define STR_INI_KEY_TOP		"top"
#define STR_INI_KEY_WIDTH	"width"
#define STR_INI_KEY_HEIGHT	"height"
#define STR_INI_KEY_STAYONTOP "stayontop"
#define STR_INI_KEY_TRANSPARENT "transparent"
#define STR_INI_KEY_MILLISECONDS "milliseconds"
#define STR_INI_KEY_TOPZEROS "topzeros"

#define AC_SRC_ALPHA		0x00000001
#define ULW_ALPHA			0x00000002

const int BORDER_WIDTH = 4; ///< Width of sizing border

const int POS_LEFTTOP = 0;		///< Return value of getWhatPos()
const int POS_LEFT = 1;			///< Return value of getWhatPos()
const int POS_LEFTBOTTOM = 2;	///< Return value of getWhatPos()
const int POS_TOP = 3;			///< Return value of getWhatPos()
const int POS_BOTTOM = 4;		///< Return value of getWhatPos()
const int POS_RIGHTTOP = 5;		///< Return value of getWhatPos()
const int POS_RIGHT = 6;		///< Return value of getWhatPos()
const int POS_RIGHTBOTTOM = 7;	///< Return value of getWhatPos()
const int POS_CAPTION = 8;		///< Return value of getWhatPos()
const int POS_CLIENT = 9;		///< Return value of getWhatPos()

const int INITIAL_WIDTH = 298;	///< Initial width of the window
const int INITIAL_HEIGHT = 100;	///< Initial height of the window

const double CAPTION_MARGIN_H = 0.05;	///< Left and right margin of caption / client width
const double CAPTION_MARGIN_V = 0.01;	///< Top and bottom margin of caption / client width
const double CAPTION_HEIGHT = 0.06;		///< Height of the caption bar / client width
const double FRAME_WIDTH  = 0.005;		///< Width of the frame / client width
const double MARGIN_LED   = 0.01;		///< Margin of the LED area / client width

const double MARGIN_DIGIT = 0.2;		///< Gap of the LED digit / digit width

const int SHADOW_LENGTH = 1;			///< Size of the shadow of the LED digit

const COLORREF COLOR_LED	 = RGB(255,255,255);	///< Color of the LED digit
const COLORREF COLOR_BACKLED = RGB(128,128,128);	///< Color of the shadow of the LED digit

const COLORREF COLOR_FRAME_LEFTTOP = RGB(128,128,128);	///< Color of window frame
const COLORREF COLOR_FRAME_RIGHTBOTTOM = RGB(64,64,64);	///< Color of window frame
const COLORREF COLOR_FRAME_CLIENT = RGB(96,96,96);		///< Color of window frame

const COLORREF COLOR_LEDAREA_LEFTTOP = RGB(16,16,16);		///< Color of LED area
const COLORREF COLOR_LEDAREA_RIGHTBOTTOM = RGB(192,192,192);///< Color of LED area
const COLORREF COLOR_LEDAREA_CLIENT = RGB(16,16,16);		///< Color of LED area

const COLORREF COLOR_CAPTION_LEFTTOP = RGB(255,255,255);	///< Gradation color of caption bar
const COLORREF COLOR_CAPTION_RIGHTTOP = RGB(255,255,255);	///< Gradation color of caption bar
const COLORREF COLOR_CAPTION_LEFTBOTTOM = RGB(32,32,32);	///< Gradation color of caption bar
const COLORREF COLOR_CAPTION_RIGHTBOTTOM = RGB(32,32,32);	///< Gradation color of caption bar

const COLORREF COLOR_TEXT_LEFTTOP = RGB(0,0,0);				///< Color of the title text
const COLORREF COLOR_TEXT_RIGHTBOTTOM = RGB(208,208,208);	///< Color of the title text

const BYTE LED_TRANSPARENCY = (BYTE)(255*70/100);

/**
	Fׂ̈̊֐B
	01 : 0    phase  1/6
	1    : 1/6  phase  3/6
	10 : 3/6  phase  4/6
	0    : 4/6  phase  1
	
	@param phase [in]ʑ(1.0)
	@return 0.0`1.0̒l
*/
double colorFunc(double phase)
{
	phase = phase - (int)phase;
	if(phase < 1.0/6.0){
		return phase*6.0;
	}else if(phase < 3.0/6.0){
		return 1.0;
	}else if(phase < 4.0/6.0){
		return (4.0/6.0-phase)*6.0;
	}else{
		return 0.0;
	}
}

/**
	l`悷B
	@param x [in]xW
	@param y [in]yW
	@param f [in]{
	@param n [in]l
*/
void drawLEDDigit(int x, int y, double f, int n)
{
	POINT pts[7][LEDDIGIT_VERTEX*2];
	int i;
	int j;
	
	for(i=0; i<7; i++){
		for(j=0; j<LEDDIGIT_VERTEX; j++){
			pts[i][j].x = (LONG)(LEDPART[i][j*2]*f + x);
			pts[i][j].y = (LONG)(LEDPART[i][j*2+1]*f + y);
		}
	}
	
	switch(n){
	case 0:
		Polygon(hdcback, pts[0], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[1], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[2], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[3], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[4], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[6], LEDDIGIT_VERTEX);
		break;
	case 1:
		Polygon(hdcback, pts[1], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[3], LEDDIGIT_VERTEX);
		break;
	case 2:
		Polygon(hdcback, pts[1], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[2], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[4], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[5], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[6], LEDDIGIT_VERTEX);
		break;
	case 3:
		Polygon(hdcback, pts[1], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[3], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[4], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[5], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[6], LEDDIGIT_VERTEX);
		break;
	case 4:
		Polygon(hdcback, pts[0], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[1], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[3], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[5], LEDDIGIT_VERTEX);
		break;
	case 5:
		Polygon(hdcback, pts[0], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[3], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[4], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[5], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[6], LEDDIGIT_VERTEX);
		break;
	case 6:
		Polygon(hdcback, pts[0], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[2], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[3], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[4], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[5], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[6], LEDDIGIT_VERTEX);
		break;
	case 7:
		Polygon(hdcback, pts[0], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[1], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[3], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[4], LEDDIGIT_VERTEX);
		break;
	case 8:
		Polygon(hdcback, pts[0], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[1], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[2], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[3], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[4], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[5], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[6], LEDDIGIT_VERTEX);
		break;
	case 9:
		Polygon(hdcback, pts[0], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[1], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[3], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[4], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[5], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[6], LEDDIGIT_VERTEX);
		break;
	default: // show E as Error
		Polygon(hdcback, pts[0], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[2], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[4], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[5], LEDDIGIT_VERTEX);
		Polygon(hdcback, pts[6], LEDDIGIT_VERTEX);
		break;
	}
}


/**
	̐オ悤ȋ``悷B
	@param rect [in]`
	@param borderwidth [in]̕
	@param lefttop [in]̐F
	@param rightbottom [in]E̐F
	@param client [in]Ŝ̐F
*/
void fillRidgeRect(const RECT& rect, int borderwidth, COLORREF lefttop, COLORREF rightbottom, COLORREF client)
{
	int x;
	int y;
	int h = rect.bottom-rect.top;
	int w = rect.right-rect.left;
	int widthbytes = (dibsection.dsBm.bmWidthBytes+3)/4*4;
	int bfr = 0xff0000;//dibsection.dsBitfields[0];
	int bfg = 0x00ff00;//dibsection.dsBitfields[1];
	int bfb = 0x0000ff;//dibsection.dsBitfields[2];
	DWORD lt = (DWORD)(bfr*(GetRValue(lefttop)/255.0))&bfr | (DWORD)(bfg*(GetGValue(lefttop)/255.0))&bfg | (DWORD)(bfb*(GetBValue(lefttop)/255.0))&bfb;
	DWORD rb = (DWORD)(bfr*(GetRValue(rightbottom)/255.0))&bfr | (DWORD)(bfg*(GetGValue(rightbottom)/255.0))&bfg | (DWORD)(bfb*(GetBValue(rightbottom)/255.0))&bfb;
	DWORD c =  (DWORD)(bfr*(GetRValue(client)/255.0))&bfr | (DWORD)(bfg*(GetGValue(client)/255.0))&bfg | (DWORD)(bfb*(GetBValue(client)/255.0))&bfb;
	
	GdiFlush();
	for(y=0; y<h; y++){
		for(x=0; x<w; x++){
			DWORD color;
			if(x<borderwidth){
				if(y>=h-x){
					color = rb;
				}else{
					color = lt;
				}
			}else if(x>=w-borderwidth){
				if(y<w-x){
					color = lt;
				}else{
					color = rb;
				}
			}else if(y<borderwidth){
				color = lt;
			}else if(y>=h-borderwidth){
				color = rb;
			}else{
				color = c;
			}
			if((x+rect.left)>=0 && (x+rect.left)<winx &&
					(y+rect.top)>=0 && (y+rect.top)<winy)
				*(DWORD*)(dibbits + (x+rect.left)*4 + (y+rect.top)*widthbytes) = color;
		}
	}
}

/**
	Of[V``悷B
	@param rect [in]`
	@param lefttop [in]̐F
	@param righttop [in]E̐F
	@param leftbottom [in]̐F
	@param rightbottom [in]E̐F
*/
void fillGradationRect(const RECT& rect, COLORREF lefttop, COLORREF righttop, COLORREF leftbottom, COLORREF rightbottom)
{
	int x;
	int y;
	int w = rect.right - rect.left;
	int h = rect.bottom - rect.top;
	int widthbytes = (dibsection.dsBm.bmWidthBytes+3)/4*4;
	int bfr = 0xff0000;//dibsection.dsBitfields[0];
	int bfg = 0x00ff00;//dibsection.dsBitfields[1];
	int bfb = 0x0000ff;//dibsection.dsBitfields[2];
	double c[4][3];
	const int R = 0;
	const int G = 1;
	const int B = 2;
	const int LT = 0;
	const int RT = 1;
	const int LB = 2;
	const int RB = 3;
	c[LT][R] = GetRValue(lefttop)/255.0;
	c[LT][G] = GetGValue(lefttop)/255.0;
	c[LT][B] = GetBValue(lefttop)/255.0;
	c[RT][R] = GetRValue(righttop)/255.0;
	c[RT][G] = GetGValue(righttop)/255.0;
	c[RT][B] = GetBValue(righttop)/255.0;
	c[LB][R] = GetRValue(leftbottom)/255.0;
	c[LB][G] = GetGValue(leftbottom)/255.0;
	c[LB][B] = GetBValue(leftbottom)/255.0;
	c[RB][R] = GetRValue(rightbottom)/255.0;
	c[RB][G] = GetGValue(rightbottom)/255.0;
	c[RB][B] = GetBValue(rightbottom)/255.0;
	
	GdiFlush();
	if( lefttop==righttop && leftbottom==rightbottom ){
		double hinv = 1.0/h;
		double yc[3];
		yc[R] = c[LT][R];
		yc[G] = c[LT][G];
		yc[B] = c[LT][B];
		double dyc[3];
		dyc[R] = (c[LB][R] - c[LT][R])*hinv;
		dyc[G] = (c[LB][G] - c[LT][G])*hinv;
		dyc[B] = (c[LB][B] - c[LT][B])*hinv;
		for(y=0; y<h; y++){
			DWORD color = (DWORD)(bfr*yc[R])&bfr | (DWORD)(bfg*yc[G])&bfg | (DWORD)(bfb*yc[B])&bfb;
			for(x=0; x<w; x++){
				if((x+rect.left)>=0 && (x+rect.left)<winx &&
						(y+rect.top)>=0 && (y+rect.top)<winy)
					*(DWORD*)(dibbits + (x+rect.left)*4 + (y+rect.top)*widthbytes) = color;
			}
			yc[R] += dyc[R];
			yc[G] += dyc[G];
			yc[B] += dyc[B];
		}
	}
	else{
		double hinv = 1.0/h;
		double winv = 1.0/w;
		
		double ycleft[3];
		ycleft[R] = c[LT][R];
		ycleft[G] = c[LT][G];
		ycleft[B] = c[LT][B];
		double ycright[3];
		ycright[R] = c[RT][R];
		ycright[G] = c[RT][G];
		ycright[B] = c[RT][B];
		double dycleft[3];
		dycleft[R] = (c[LB][R] - c[LT][R])*hinv;
		dycleft[G] = (c[LB][G] - c[LT][G])*hinv;
		dycleft[B] = (c[LB][B] - c[LT][B])*hinv;
		double dycright[3];
		dycright[R] = (c[RB][R] - c[RT][R])*hinv;
		dycright[G] = (c[RB][G] - c[RT][G])*hinv;
		dycright[B] = (c[RB][B] - c[RT][B])*hinv;
		for(y=0; y<h; y++){
			double xc[3];
			xc[R] = ycleft[R];
			xc[G] = ycleft[G];
			xc[B] = ycleft[B];
			double dxc[3];
			dxc[R] = (ycright[R] - ycleft[R])*winv;
			dxc[G] = (ycright[G] - ycleft[G])*winv;
			dxc[B] = (ycright[B] - ycleft[B])*winv;
			for(x=0; x<w; x++){
				DWORD* pcolor = (DWORD*)(dibbits + (y+rect.top)*widthbytes + (x+rect.left)*4);
				if((x+rect.left)>=0 && (x+rect.left)<winx &&
						(y+rect.top)>=0 && (y+rect.top)<winy)
					*pcolor = (DWORD)(bfr*xc[R])&bfr | (DWORD)(bfg*xc[G])&bfg | (DWORD)(bfb*xc[B])&bfb;
				xc[R] += dxc[R];
				xc[G] += dxc[G];
				xc[B] += dxc[B];
			}
			ycleft[R] += dycleft[R];
			ycleft[G] += dycleft[G];
			ycleft[B] += dycleft[B];
			ycright[R] += dycright[R];
			ycright[G] += dycright[G];
			ycright[B] += dycright[B];
		}
	}
}

/**
	`̃At@`lݒ肷B
	̊֐ŋ`̃At@lvalueɐݒ肳A
	eFAt@lB
	
	alpha傤127̎A኱ZB
	
	@param rect [in]`
	@param alpha [in]At@l
	@param alphaonly [in]At@lςꍇTRUE
*/
void fillAlphaRect(const RECT& rect, DWORD alpha, BOOL alphaonly)
{
	int x;
	int y;
	int w = rect.right - rect.left;
	int h = rect.bottom - rect.top;
	int widthbytes = (dibsection.dsBm.bmWidthBytes+3)/4*4;
	int bfr = 0xff0000;//dibsection.dsBitfields[0];
	int bfg = 0x00ff00;//dibsection.dsBitfields[1];
	int bfb = 0x0000ff;//dibsection.dsBitfields[2];
	DWORD alphabit = alpha<<24;
	GdiFlush();
	if((alpha == 255) || alphaonly){
		for(y=0; y<h; y++){
			for(x=0; x<w; x++){
				DWORD* pcolor = (DWORD*)(dibbits + (x+rect.left)*4 + (y+rect.top)*widthbytes);
				if((x+rect.left)>=0 && (x+rect.left)<winx &&
						(y+rect.top)>=0 && (y+rect.top)<winy)
					*pcolor |= 0xff000000;
			}
		}
	}
	else if(alpha == 127){
		for(y=0; y<h; y++){
			for(x=0; x<w; x++){
				if((x+rect.left)>=0 && (x+rect.left)<winx &&
						(y+rect.top)>=0 && (y+rect.top)<winy) {
					DWORD* pcolor = (DWORD*)(dibbits + (x+rect.left)*4 + (y+rect.top)*widthbytes);
					DWORD color = *pcolor;
					*pcolor = alphabit | ((bfr&color)>>1)&bfr | ((bfg&color)>>1)&bfg | ((bfb&color)>>1)&bfb;
				}
			}
		}
	}
	else{
		for(y=0; y<h; y++){
			for(x=0; x<w; x++){
				if((x+rect.left)>=0 && (x+rect.left)<winx &&
						(y+rect.top)>=0 && (y+rect.top)<winy) {
					DWORD* pcolor = (DWORD*)(dibbits + (x+rect.left)*4 + (y+rect.top)*widthbytes);
					DWORD color = *pcolor;
					*pcolor = alphabit |
					(DWORD)((bfr&color)*alpha>>8)&bfr |	// ۂ255Ŋ̂BA
					(DWORD)((bfg&color)*alpha>>8)&bfg | // ̈8rbgVtg(256Ŋɓ)őpB
					(DWORD)((bfb&color)*alpha>>8)&bfb;
				}
			}
		}
	}
}

/**
	c莞Ԃ`悷B
	@param lefttime [in]c莞
*/
void drawLeftTime(const SYSTEMTIME& tm)
{
	RECT crect;
	RECT ledrect;
	RECT leddigitrect;
	RECT timebarrect;
	unsigned int lefttime;
	unsigned int x;
	int blockx;
	int blocky;
	double f;
	int i;
	int j;
	bool d;
	RECT blockrect;
	RECT textrect;
	RECT titlerect;
	char str[32];
	int phase;
	const double period = 15000;
	const double phasegap = 1.0/6.0;
	int ms;
	
	GetClientRect(hwnd, &crect);
	fillRidgeRect(crect, 1, COLOR_FRAME_LEFTTOP, COLOR_FRAME_RIGHTBOTTOM, COLOR_FRAME_CLIENT);
	ledrect.left = crect.right*FRAME_WIDTH;
	ledrect.right = crect.right*(1.0-FRAME_WIDTH);
	ledrect.top = crect.right*(FRAME_WIDTH+CAPTION_HEIGHT);
	ledrect.bottom = crect.bottom - crect.right*FRAME_WIDTH;
	fillRidgeRect(ledrect, 1, COLOR_LEDAREA_LEFTTOP, COLOR_LEDAREA_RIGHTBOTTOM, COLOR_LEDAREA_CLIENT);
	timebarrect = ledrect;
	InflateRect(&timebarrect, -1, -1);
	lefttime = (23-tm.wHour)*3600 + (59-tm.wMinute)*60 + 59-tm.wSecond;
	ms=tm.wMilliseconds;
	if(ms==0) {
		lefttime++;
		ms=999;
	}
	timebarrect.right = timebarrect.left + (timebarrect.right-timebarrect.left)*lefttime/86400;
	phase = lefttime*1000 - ms;
	fillGradationRect(timebarrect,
		RGB(colorFunc(phase/period)*255, colorFunc(phase/period + 1.0/3.0)*255, colorFunc(phase/period + 2.0/3.0)*255),
		RGB(colorFunc(phase/period + phasegap)*255, colorFunc(phase/period + phasegap + 1.0/3.0)*255, colorFunc(phase/period + phasegap + 2.0/3.0)*255),
		RGB(colorFunc(phase/period + phasegap*2)*255, colorFunc(phase/period + phasegap*2 + 1.0/3.0)*255, colorFunc(phase/period + phasegap*2 + 2.0/3.0)*255),
		RGB(colorFunc(phase/period + phasegap*3)*255, colorFunc(phase/period + phasegap*3 + 1.0/3.0)*255, colorFunc(phase/period + phasegap*3 + 2.0/3.0)*255));
	leddigitrect.left = crect.right*(FRAME_WIDTH+MARGIN_LED);
	leddigitrect.right = crect.right*(1.0-FRAME_WIDTH-MARGIN_LED);
	leddigitrect.top = crect.right*(FRAME_WIDTH+MARGIN_LED+CAPTION_HEIGHT);
	leddigitrect.bottom = crect.bottom - crect.right*(FRAME_WIDTH+MARGIN_LED);
	f = (leddigitrect.right - leddigitrect.left)/((6.0+5.5*MARGIN_DIGIT)*LEDDIGIT_WIDTH);
	for(i=0, x=1, d=FALSE; i<5; i++){
		if(x==1 || showtopzeros || lefttime>=x) {
			SelectObject(hdcback, hbackledbrush);
			drawLEDDigit(leddigitrect.left+SHADOW_LENGTH + f*(1+MARGIN_DIGIT)*LEDDIGIT_WIDTH*(4-i), leddigitrect.top+SHADOW_LENGTH,
			 f, (lefttime%(x*10))/x);
			SelectObject(hdcback, hledbrush);
			drawLEDDigit(leddigitrect.left + f*(1+MARGIN_DIGIT)*LEDDIGIT_WIDTH*(4-i), leddigitrect.top,
			 f, (lefttime%(x*10))/x);
			 d=TRUE;
		}
		x*=10;
	}
	if(showmilliseconds){
		for(i=0, x=10; i<2; i++){
			SelectObject(hdcback, hbackledbrush);
			drawLEDDigit(leddigitrect.left + SHADOW_LENGTH + f*(1+MARGIN_DIGIT)*LEDDIGIT_WIDTH*5 + 0.5*f*(1+MARGIN_DIGIT)*LEDDIGIT_WIDTH*(1-i),
			 leddigitrect.top + SHADOW_LENGTH + 0.5*f*LEDDIGIT_HEIGHT, 0.5*f, ((999-ms)%(x*10))/x);
			SelectObject(hdcback, hledbrush);
			drawLEDDigit(leddigitrect.left + f*(1+MARGIN_DIGIT)*LEDDIGIT_WIDTH*5 + 0.5*f*(1+MARGIN_DIGIT)*LEDDIGIT_WIDTH*(1-i),
			 leddigitrect.top + 0.5*f*LEDDIGIT_HEIGHT, 0.5*f, ((999-ms)%(x*10))/x);
			x*=10;
		}
	}
	else{
		RECT secrect;
		sprintf(str, "b");
		SelectObject(hdcback, hsecfont);
		SetTextColor(hdcback, COLOR_BACKLED);
		secrect.left = leddigitrect.left + SHADOW_LENGTH + f*(1+MARGIN_DIGIT)*LEDDIGIT_WIDTH*5;
		secrect.top = leddigitrect.top + SHADOW_LENGTH + 0.5*f*LEDDIGIT_HEIGHT;
		secrect.right = leddigitrect.right;
		secrect.bottom = leddigitrect.bottom;
		DrawText(hdcback, str, -1, &secrect, DT_LEFT|DT_TOP|DT_SINGLELINE);
		SetTextColor(hdcback, COLOR_LED);
		OffsetRect(&secrect, -SHADOW_LENGTH, -SHADOW_LENGTH);
		DrawText(hdcback, str, -1, &secrect, DT_LEFT|DT_TOP|DT_SINGLELINE);
	}
	SetRect(&titlerect,
	 crect.right*(FRAME_WIDTH),
	 crect.right*FRAME_WIDTH,
	 crect.right*(1.0-FRAME_WIDTH),
	 crect.right*(FRAME_WIDTH+CAPTION_HEIGHT));
	fillGradationRect(titlerect, COLOR_CAPTION_LEFTTOP, COLOR_CAPTION_RIGHTTOP, COLOR_CAPTION_LEFTBOTTOM, COLOR_CAPTION_RIGHTBOTTOM);
	textrect = titlerect;
	InflateRect(&textrect, -crect.right*CAPTION_MARGIN_H, -crect.right*CAPTION_MARGIN_V);
	sprintf(str, "%dN%d%dI", tm.wYear, tm.wMonth, tm.wDay);
	SelectObject(hdcback, hcaptionfont);
	SetTextColor(hdcback, COLOR_TEXT_LEFTTOP);
	DrawText(hdcback, str, -1, &textrect, DT_LEFT|DT_VCENTER|DT_SINGLELINE);
	OffsetRect(&textrect, 1, 1);
	SetTextColor(hdcback, COLOR_TEXT_RIGHTBOTTOM);
	DrawText(hdcback, str, -1, &textrect, DT_LEFT|DT_VCENTER|DT_SINGLELINE);
	
	if(islayered){
		fillAlphaRect(crect, 255, TRUE);
		//fillAlphaRect(ledrect, 127, FALSE);
		fillAlphaRect(ledrect, alphacurrent, FALSE);
	}
	backpainted = true;
}

/**
	hdcbackAyтɊ֘AÂꂽIuWFNgjB
*/
void releaseDCBack()
{
	DeleteObject(SelectObject(hdcback, hbmporg));
	SelectObject(hdcback, hfontorg);
	SelectObject(hdcback, hbrushorg);
	SelectObject(hdcback, hpenorg);
	DeleteDC(hdcback);
	hdcback = 0;
	DeleteObject(hsecfont);
	DeleteObject(hcaptionfont);
}

/**
	hdcbackƊ֘AIuWFNg쐬B
*/
void createDCBack()
{
	RECT crect;
	HDC hdc;
	HBITMAP hbmp;
	BITMAPINFO bmi;
	
	if(!GetClientRect(hwnd, &crect)){
		crect.left = 0;
		crect.top = 0;
		crect.right = GetPrivateProfileInt(STR_INI_APPNAME, STR_INI_KEY_WIDTH, INITIAL_WIDTH, inifilename);
		crect.bottom = GetPrivateProfileInt(STR_INI_APPNAME, STR_INI_KEY_HEIGHT, INITIAL_HEIGHT, inifilename);
	}
	printf("Rect: %d, %d, %d, %d\n", crect.left, crect.top, crect.right, crect.bottom);
	hdc = GetDC(NULL);
	assert(hdc);
	memset(&bmi, 0, sizeof(BITMAPINFO));
	bmi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	bmi.bmiHeader.biWidth = crect.right;
	bmi.bmiHeader.biHeight = -crect.bottom;
	bmi.bmiHeader.biPlanes = 1;
	bmi.bmiHeader.biBitCount = 32;
	bmi.bmiHeader.biCompression = BI_RGB;
	bmi.bmiHeader.biSizeImage = 0;
	bmi.bmiHeader.biXPelsPerMeter = 72*254;
	bmi.bmiHeader.biYPelsPerMeter = 72*254;
	bmi.bmiHeader.biClrUsed = 0;
	bmi.bmiHeader.biClrImportant = 0;
	hbmp = CreateDIBSection(hdc, &bmi, DIB_RGB_COLORS, (void**)&dibbits, NULL, 0);
	assert(hbmp);
	assert(dibbits);
	winx=crect.right;
	winy=crect.bottom;
	GetObject(hbmp, sizeof(DIBSECTION), &dibsection);
	hdcback = CreateCompatibleDC(hdc);
	assert(hdcback);
	ReleaseDC(NULL, hdc);
	hbmporg = (HBITMAP)SelectObject(hdcback, hbmp);
	assert(hbmporg);
	hfontorg = (HFONT)GetCurrentObject(hdcback, OBJ_FONT);
	assert(hfontorg);
	hbrushorg = (HBRUSH)GetCurrentObject(hdcback, OBJ_BRUSH);
	assert(hbrushorg);
	hpenorg = (HPEN)GetCurrentObject(hdcback, OBJ_PEN);
	assert(hpenorg);
	hcaptionfont = CreateFont(
		crect.right*(CAPTION_HEIGHT-2*CAPTION_MARGIN_V), // tHg̍
		0,                      // ϕ
		0,                      // ̊px
		0,                      // x[XC̊px
		FW_NORMAL,              // tHg̑
		FALSE,                  // Α̂ɂ邩ǂ
		FALSE,                  // t邩ǂ
		FALSE,                  // t邩ǂ
		DEFAULT_CHARSET,        // Zbg̎ʎq
		OUT_DEFAULT_PRECIS,     // o͐x
		CLIP_DEFAULT_PRECIS,    // NbsOx
		DEFAULT_QUALITY,        // o͕i
		FIXED_PITCH|FF_MODERN,  // sb`ƃt@~
		NULL);                  // tHg
	assert(hcaptionfont);
	hsecfont = CreateFont(
		0.5*LEDDIGIT_HEIGHT*(crect.right*(1.0-2*(FRAME_WIDTH*MARGIN_LED))/((6.0+5.5*MARGIN_DIGIT)*LEDDIGIT_WIDTH)), // tHg̍
		0,                      // ϕ
		0,                      // ̊px
		0,                      // x[XC̊px
		FW_BOLD,                // tHg̑
		FALSE,                  // Α̂ɂ邩ǂ
		FALSE,                  // t邩ǂ
		FALSE,                  // t邩ǂ
		DEFAULT_CHARSET,        // Zbg̎ʎq
		OUT_DEFAULT_PRECIS,     // o͐x
		CLIP_DEFAULT_PRECIS,    // NbsOx
		DEFAULT_QUALITY,        // o͕i
		FIXED_PITCH|FF_ROMAN,   // sb`ƃt@~
		NULL);                  // tHg
	assert(hsecfont);
	SetBkMode(hdcback, TRANSPARENT);
	SelectObject(hdcback, GetStockObject(NULL_PEN));
}

/**
	}EXJ[\̂ꏊ̑ԂB
	@param x [in]J[\̃NCAgWłx
	@param y [in]J[\̃NCAgWły
	@return POS_LEFTTOP, POS_LEFT, POS_LEFTBOTTOM, POS_TOP, POS_CAPTION, POS_CLIENT, POS_BOTTOM
	 POS_RIGHTTOP, POS_RIGHT, POS_RIGHTBOTTOM
*/
int getWhatPos(int x, int y)
{
	RECT crect;
	GetClientRect(hwnd, &crect);
	
	if(x<BORDER_WIDTH){
		if(y<BORDER_WIDTH){
			return POS_LEFTTOP;
		}else if(y>=crect.bottom-BORDER_WIDTH){
			return POS_LEFTBOTTOM;
		}else{
			return POS_LEFT;
		}
	}else if(x>=crect.right-BORDER_WIDTH){
		if(y<BORDER_WIDTH){
			return POS_RIGHTTOP;
		}else if(y>=crect.bottom-BORDER_WIDTH){
			return POS_RIGHTBOTTOM;
		}else{
			return POS_RIGHT;
		}
	}else{
		if(y<BORDER_WIDTH){
			return POS_TOP;
		}else if(y<CAPTION_HEIGHT){
			return POS_CAPTION;
		}else if(y>=crect.bottom-BORDER_WIDTH){
			return POS_BOTTOM;
		}else{
			return POS_CLIENT;
		}
	}
}

/**
	j[̃`FbN؂ւB
	@param hmenu [in]j[nh
	@param id [in]j[ID
	@param checked [in]`FbNȂtrueA`FbNȂfalse
*/
void checkMenu(HMENU hmenu, UINT id, bool checked)
{
	MENUITEMINFO mmi;
	memset(&mmi, 0, sizeof(mmi));
	mmi.cbSize = sizeof(mmi);
	mmi.fMask = MIIM_STATE;
	mmi.fState = checked?MFS_CHECKED:MFS_UNCHECKED;
	SetMenuItemInfo(hmenu, id, FALSE, &mmi);
}

/**
	CEBhẼEBhEvV[WB
	@param hwnd [in]EBhEnhB
	@param umsg [in]bZ[WIDB
	@param wparam [in]ǉp[^W
	@param lparam [in]ǉp[^L
	@return bZ[WAʏ0ԂB
*/
LRESULT CALLBACK myWindowProcedure(HWND hwnd, UINT umsg, WPARAM wparam, LPARAM lparam)
{
	switch(umsg){
	case WM_SIZE:
	{
		if(wparam == SIZE_MINIMIZED) return 0;
		if(hdcback) releaseDCBack();
		createDCBack();
		return 0;
	}
	case WM_PAINT:
	{
		if(!hdcback) return 0;
		PAINTSTRUCT ps;
		HDC hdc;
		hdc = BeginPaint(hwnd, &ps);
		if(hdc){
			if(backpainted){
				RECT crect;
				GetClientRect(hwnd, &crect);
				if(islayered){
					SIZE size;
					size.cx = crect.right;
					size.cy = crect.bottom;
					POINT pt = {0,0};
					BLENDFUNCTION bf;
					bf.BlendOp = AC_SRC_OVER;
					bf.BlendFlags = 0;
					bf.SourceConstantAlpha = 0xff;
					bf.AlphaFormat = AC_SRC_ALPHA;
					UpdateLayeredWindow(hwnd, NULL, NULL, &size, hdcback, &pt, 0, &bf, ULW_ALPHA);
				}
				else{
					BitBlt(hdc, 0, 0, crect.right, crect.bottom, hdcback, 0, 0, SRCCOPY);
				}
			}
			EndPaint(hwnd, &ps);
		}
		return 0;
	}
	case WM_TIMER:
	{
		assert(wparam == TID_MAIN);
		if(!hdcback) return 0;
		
		if(alphapurpose < alphacurrent) alphacurrent--;
		else if(alphapurpose > alphacurrent) alphacurrent++;
		if(alphacurrent == 255){
			islayered = false;
			SetWindowLong(hwnd, GWL_EXSTYLE, GetWindowLong(hwnd, GWL_EXSTYLE)&~WS_EX_LAYERED);
			checkMenu(hnotifymenu, MID_TRANSPARENT, FALSE);
		}
		if(showmilliseconds || (previoustimer/100 != GetTickCount()/100)){
			SYSTEMTIME tm;
			GetLocalTime(&tm);
			drawLeftTime(tm);
			RedrawWindow(hwnd, NULL, NULL, RDW_INTERNALPAINT|RDW_INVALIDATE);
			previoustimer = GetTickCount();
		}
		return 0;
	}
	case WM_KEYDOWN:
		return 0;
	case WM_LBUTTONDOWN:
		//SetCapture(hwnd);
		switch(getWhatPos(GET_X_LPARAM(lparam), GET_Y_LPARAM(lparam))){
		case POS_LEFTTOP:
			SendMessage(hwnd, WM_SYSCOMMAND, SC_SIZE|4, 0); break;
		case POS_LEFT:
			SendMessage(hwnd, WM_SYSCOMMAND, SC_SIZE|1, 0); break;
		case POS_LEFTBOTTOM:
			SendMessage(hwnd, WM_SYSCOMMAND, SC_SIZE|7, 0); break;
		case POS_TOP:
			SendMessage(hwnd, WM_SYSCOMMAND, SC_SIZE|3, 0); break;
		case POS_BOTTOM:
			SendMessage(hwnd, WM_SYSCOMMAND, SC_SIZE|6, 0); break;
		case POS_RIGHTTOP:
			SendMessage(hwnd, WM_SYSCOMMAND, SC_SIZE|5, 0); break;
		case POS_RIGHT:
			SendMessage(hwnd, WM_SYSCOMMAND, SC_SIZE|2, 0); break;
		case POS_RIGHTBOTTOM:
			SendMessage(hwnd, WM_SYSCOMMAND, SC_SIZE|8, 0); break;
		case POS_CAPTION:
		case POS_CLIENT:
			SendMessage(hwnd, WM_SYSCOMMAND, SC_MOVE|2, 0); break;
		default:
			assert(0);
		}
		return 0;
	case WM_RBUTTONDOWN:
		SetCapture(hwnd);
		return 0;
	case WM_MOUSEMOVE:
		switch(getWhatPos(GET_X_LPARAM(lparam), GET_Y_LPARAM(lparam))){
		case POS_LEFTTOP:
			SetCursor(LoadCursor(NULL, IDC_SIZENWSE)); break;
		case POS_LEFT:
			SetCursor(LoadCursor(NULL, IDC_SIZEWE)); break;
		case POS_LEFTBOTTOM:
			SetCursor(LoadCursor(NULL, IDC_SIZENESW)); break;
		case POS_TOP:
			SetCursor(LoadCursor(NULL, IDC_SIZENS)); break;
		case POS_BOTTOM:
			SetCursor(LoadCursor(NULL, IDC_SIZENS)); break;
		case POS_RIGHTTOP:
			SetCursor(LoadCursor(NULL, IDC_SIZENESW)); break;
		case POS_RIGHT:
			SetCursor(LoadCursor(NULL, IDC_SIZEWE)); break;
		case POS_RIGHTBOTTOM:
			SetCursor(LoadCursor(NULL, IDC_SIZENWSE)); break;
		case POS_CAPTION:
		case POS_CLIENT:
			SetCursor(LoadCursor(NULL, IDC_ARROW)); break;
		default:
			assert(0);
		}
		return 0;
	case WM_LBUTTONUP:
		ReleaseCapture();
		return 0;
	case WM_RBUTTONUP:
		POINT pt;
		GetCursorPos(&pt);
		TrackPopupMenu(hnotifymenu, 0, pt.x, pt.y, 0, hwnd, 0);
		ReleaseCapture();
		return 0;
	case WM_SIZING:
	{
		LPRECT prect = (LPRECT)lparam;
		RECT rectorg;
		double prefer_width = ((6.0+5.5*MARGIN_DIGIT)*LEDDIGIT_WIDTH)/(1.0-2*(FRAME_WIDTH+MARGIN_LED));
		double prefer_height = LEDDIGIT_HEIGHT + prefer_width*(2*(FRAME_WIDTH+MARGIN_LED)+CAPTION_HEIGHT);
		double prefer_aspect = prefer_height/prefer_width;
		
		rectorg = *prect;
		OffsetRect(prect, -prect->left, -prect->top);
		switch(wparam){
		case WMSZ_BOTTOM:
		case WMSZ_TOP:
			prect->right = prect->bottom/prefer_aspect;
			break;
		case WMSZ_LEFT:
		case WMSZ_RIGHT:
			prect->bottom = prect->right*prefer_aspect;
			break;
		default:
			if(prect->bottom < prect->right*prefer_aspect){
				prect->right = prect->bottom/prefer_aspect;
			}else{
				prect->bottom = prect->right*prefer_aspect;
			}
		}
		
		switch(wparam){
		case WMSZ_BOTTOM:
			OffsetRect(prect, rectorg.left, rectorg.top); break;
		case WMSZ_BOTTOMLEFT:
			OffsetRect(prect, rectorg.right-prect->right, rectorg.top); break;
		case WMSZ_BOTTOMRIGHT:
			OffsetRect(prect, rectorg.left, rectorg.top); break;
		case WMSZ_LEFT:
			OffsetRect(prect, rectorg.right-prect->right, rectorg.top); break;
		case WMSZ_RIGHT:
			OffsetRect(prect, rectorg.left, rectorg.top); break;
		case WMSZ_TOP:
			OffsetRect(prect, rectorg.left, rectorg.bottom-prect->bottom); break;
		case WMSZ_TOPLEFT:
			OffsetRect(prect, rectorg.right-prect->right, rectorg.bottom-prect->bottom); break;
		case WMSZ_TOPRIGHT:
			OffsetRect(prect, rectorg.left, rectorg.bottom-prect->bottom); break;
		default:
			assert(0);
		}
		return TRUE;
	}
	case WM_GETMINMAXINFO:
	{
		LPMINMAXINFO pmmi = (LPMINMAXINFO)lparam;
		
		pmmi->ptMinTrackSize.x = 10;
		pmmi->ptMinTrackSize.y = 10;
		
		return 0;
	}
	case WM_COMMAND:
		printf("Command: %u, %u\n", wparam, lparam);
		if(lparam==0){
			switch(LOWORD(wparam)){
			case MID_CLOSE:
				SendMessage(hwnd, WM_CLOSE, 0, 0);
				break;
			case MID_STAYONTOP:
				isstayontop = !isstayontop;
				SetWindowPos(hwnd, isstayontop?HWND_TOPMOST:HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOMOVE|SWP_NOSIZE);
				checkMenu(hnotifymenu, MID_STAYONTOP, isstayontop);
				break;
			case MID_TRANSPARENT:
				backpainted = false;
				if(!islayered){
					islayered = true;
					alphapurpose = LED_TRANSPARENCY;
					SetWindowLong(hwnd, GWL_EXSTYLE, (GetWindowLong(hwnd, GWL_EXSTYLE)&~WS_EX_LAYERED)|WS_EX_LAYERED);
					checkMenu(hnotifymenu, MID_TRANSPARENT, TRUE);
				}
				else{
					alphapurpose = 255;
				}
				break;
			case MID_MILLISECONDS:
				showmilliseconds = !showmilliseconds;
				SetTimer(hwnd, TID_MAIN, 10, NULL);
				checkMenu(hnotifymenu, MID_MILLISECONDS, showmilliseconds);
				break;
			case MID_SHOWTOPZEROS:
				showtopzeros = !showtopzeros;
				checkMenu(hnotifymenu, MID_SHOWTOPZEROS, showtopzeros);
				break;
			case MID_VERSION:
				MessageBox(hwnd, STR_VERSION_INFO, STR_VERSION_CAPTION, MB_OK|MB_ICONINFORMATION);
				break;
			default:
				assert(0);
			}
			return 0;
		}
		else return DefWindowProc(hwnd, umsg, wparam, lparam);
	case WM_USER_NOTIFYICON:
	{
		assert(wparam == NIID_MAIN);
		switch(lparam){
		case WM_LBUTTONDOWN:
		case WM_RBUTTONDOWN:
			SetWindowPos(hwnd, HWND_TOP, 0, 0, 0, 0, SWP_NOMOVE|SWP_NOSIZE);
			BringWindowToTop(hwnd);
			break;
		case WM_LBUTTONUP:
		case WM_RBUTTONUP:
		{
			POINT pt;
			GetCursorPos(&pt);
			TrackPopupMenu(hnotifymenu, 0, pt.x, pt.y, 0, hwnd, 0);
			break;
		}
		default:
			break;
		}
		return 0;
	}
	case WM_CREATE:
		// initialize variables
		backpainted = true;
		isstayontop = (bool)GetPrivateProfileInt(STR_INI_APPNAME, STR_INI_KEY_STAYONTOP, 0, inifilename);
		islayered = (bool)GetPrivateProfileInt(STR_INI_APPNAME, STR_INI_KEY_TRANSPARENT, 0, inifilename);
		showmilliseconds = (bool)GetPrivateProfileInt(STR_INI_APPNAME, STR_INI_KEY_MILLISECONDS, 1, inifilename);
		showtopzeros = (bool)GetPrivateProfileInt(STR_INI_APPNAME, STR_INI_KEY_TOPZEROS, 1, inifilename);
		RECT rect;
		rect.left = GetPrivateProfileInt(STR_INI_APPNAME, STR_INI_KEY_LEFT, (GetSystemMetrics(SM_CXSCREEN)-INITIAL_WIDTH)/2, inifilename);
		rect.top = GetPrivateProfileInt(STR_INI_APPNAME, STR_INI_KEY_TOP, (GetSystemMetrics(SM_CYSCREEN)-INITIAL_HEIGHT)/2, inifilename);
		rect.right = rect.left + GetPrivateProfileInt(STR_INI_APPNAME, STR_INI_KEY_WIDTH, INITIAL_WIDTH, inifilename);
		rect.bottom = rect.top + GetPrivateProfileInt(STR_INI_APPNAME, STR_INI_KEY_HEIGHT, INITIAL_HEIGHT, inifilename);
		SetWindowLong(hwnd, GWL_EXSTYLE, GetWindowLong(hwnd, GWL_EXSTYLE)|(islayered?WS_EX_LAYERED:0));
		if(islayered){
			alphapurpose = alphacurrent = LED_TRANSPARENCY;
		}
		else{
			alphapurpose = alphacurrent = 255;
		}
		SetWindowPos(hwnd, isstayontop?HWND_TOPMOST:HWND_NOTOPMOST,
		 rect.left, rect.top, rect.right-rect.left, rect.bottom-rect.top, 0);
		SetTimer(hwnd, TID_MAIN, 10, NULL);
		previoustimer = 0;
		// create gdi object
		hledbrush = CreateSolidBrush(COLOR_LED);
		assert(hledbrush);
		hbackledbrush = CreateSolidBrush(COLOR_BACKLED);
		assert(hbackledbrush);
		// create notify icon
		NOTIFYICONDATA nid;
		memset(&nid, 0, sizeof(nid));
		nid.cbSize = sizeof(nid);
		nid.hWnd = hwnd;
		nid.uID = NIID_MAIN;
		nid.uFlags = NIF_ICON|NIF_TIP|NIF_MESSAGE;
		nid.uCallbackMessage = WM_USER_NOTIFYICON;
		nid.hIcon = LoadIcon(NULL, IDI_WINLOGO);
		strncpy(nid.szTip, STR_NOTIFYICON, sizeof(nid.szTip));
		Shell_NotifyIcon(NIM_ADD, &nid);
		// create popup menu
		hnotifymenu = CreatePopupMenu();
		InsertMenu(hnotifymenu, -1, MF_BYPOSITION, MID_CLOSE, STR_MENU_CLOSE);
		InsertMenu(hnotifymenu, -1, MF_BYPOSITION, MID_STAYONTOP, STR_MENU_STAYONTOP);
		InsertMenu(hnotifymenu, -1, MF_BYPOSITION, MID_TRANSPARENT, STR_MENU_TRANSPARENT);
		InsertMenu(hnotifymenu, -1, MF_BYPOSITION, MID_MILLISECONDS, STR_MENU_MILLISECONDS);
		InsertMenu(hnotifymenu, -1, MF_BYPOSITION, MID_SHOWTOPZEROS, STR_MENU_SHOWTOPZEROS);
		InsertMenu(hnotifymenu, -1, MF_BYPOSITION, MID_VERSION, STR_MENU_VERSION);
		checkMenu(hnotifymenu, MID_STAYONTOP, isstayontop);
		checkMenu(hnotifymenu, MID_TRANSPARENT, islayered);
		checkMenu(hnotifymenu, MID_MILLISECONDS, showmilliseconds);
		checkMenu(hnotifymenu, MID_SHOWTOPZEROS, showtopzeros);
		return 0;
	case WM_CLOSE:
	{
		char str[32];
		RECT rect;
		GetWindowRect(hwnd, &rect);
		WritePrivateProfileString(STR_INI_APPNAME, STR_INI_KEY_LEFT, itoa(rect.left, str, 10), inifilename);
		WritePrivateProfileString(STR_INI_APPNAME, STR_INI_KEY_TOP, itoa(rect.top, str, 10), inifilename);
		WritePrivateProfileString(STR_INI_APPNAME, STR_INI_KEY_WIDTH, itoa(rect.right-rect.left, str, 10), inifilename);
		WritePrivateProfileString(STR_INI_APPNAME, STR_INI_KEY_HEIGHT, itoa(rect.bottom-rect.top, str, 10), inifilename);
		WritePrivateProfileString(STR_INI_APPNAME, STR_INI_KEY_STAYONTOP,  isstayontop?"1":"0", inifilename);
		WritePrivateProfileString(STR_INI_APPNAME, STR_INI_KEY_TRANSPARENT, islayered?"1":"0", inifilename);
		WritePrivateProfileString(STR_INI_APPNAME, STR_INI_KEY_MILLISECONDS, showmilliseconds?"1":"0", inifilename);
		WritePrivateProfileString(STR_INI_APPNAME, STR_INI_KEY_TOPZEROS, showtopzeros?"1":"0", inifilename);
		DestroyWindow(hwnd);
		return 0;
	}
	case WM_DESTROY:
	{
		if(hdcback) releaseDCBack();
		DeleteObject(hledbrush);
		DeleteObject(hbackledbrush);
		NOTIFYICONDATA nid;
		memset(&nid, 0, sizeof(nid));
		nid.cbSize = sizeof(nid);
		nid.hWnd = hwnd;
		nid.uID = NIID_MAIN;
		Shell_NotifyIcon(NIM_DELETE, &nid);
		DestroyMenu(hnotifymenu);
		PostQuitMessage(0);
		return 0;
	}
	default:
		return DefWindowProc(hwnd, umsg, wparam, lparam);
	}
}

/**
	AvP[Ṽ\[XB
	@return trueԂB
*/
bool initializeApplication()
{
	// ini filename
	char* pdot;
	GetModuleFileName(NULL, inifilename, sizeof(inifilename));
	pdot = strrchr(inifilename, '.');
	assert(pdot);
	strcpy(pdot, ".ini");
	
	SetPriorityClass(GetCurrentProcess(), NORMAL_PRIORITY_CLASS);
	SetPriorityClass(GetCurrentThread(), THREAD_PRIORITY_NORMAL);
	
	
	return true;
}

/**
	EBhEƊ֘A\[X쐬ďB
	O[oϐhinstanceɃvÕCX^XĂ邱ƁB
	@return trueԂB
*/
bool initializeWindow()
{
	// create a new window class
	WNDCLASSEX wcex;
	memset(&wcex, 0, sizeof(wcex));
	wcex.cbSize = sizeof(wcex);
	wcex.style = CS_OWNDC;
	wcex.lpfnWndProc = myWindowProcedure;
	wcex.cbClsExtra = 0;
	wcex.cbWndExtra = 0;
	wcex.hInstance = hinstance;
	wcex.hIcon = NULL;
	wcex.hCursor = LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground = NULL;
	wcex.lpszMenuName = NULL;
	wcex.lpszClassName = "LeftTimeWindow";
	wcex.hIconSm = NULL;
	if(!RegisterClassEx(&wcex)) return false;
	
	// create main window
	hwnd = CreateWindowEx(
		WS_EX_TOOLWINDOW,
		"LeftTimeWindow",
		"The Left Time",
		WS_POPUP,
		CW_USEDEFAULT,
		CW_USEDEFAULT,
		INITIAL_WIDTH,
		INITIAL_HEIGHT,
		NULL,
		NULL,
		hinstance,
		NULL);
	if(!hwnd) return false;
	
	// show and update the window
	ShowWindow(hwnd, SW_SHOWNORMAL);
	UpdateWindow(hwnd);
	
	return true;
}

/**
	EBhEƊ֘A\[XjB
*/
void finalizeWindow()
{
	return;
}


/**
	WindowsAvP[ṼGg|CgB
*/
int WINAPI WinMain(HINSTANCE hinst, HINSTANCE, LPTSTR, int)
{
	MSG msg;
	int ret;
	
	hinstance = hinst;	// vÕCX^Xۑ
	
	if(!initializeApplication()) return -1; // AvP[V
	if(!initializeWindow()) return -1;	// EBhE쐬
	
	// message loop
	for(;;)	// [v
	{
		ret = GetMessage(&msg, NULL, 0, 0);	// ̃bZ[W𓾂B
		if (ret == 0) break;	// WM_QUIT̂Ƃret0ɂȂ̂ŁA0ȂIB
		if (ret == -1) break;	// G[̂Ƃret-1ɂȂ̂ŁA-1ȂIB
		DispatchMessage(&msg);	// bZ[WB
	}
	
	finalizeWindow();	// EBhEj
	
	return 0;
}

/**
	R\[pGg|Cg
	@param [in]args ̔z
	@param [in]argv z̗vf
*/
int main(char** /*args*/, int /*argv*/)
{
	return WinMain(NULL, NULL, NULL, 0);
}
